@@ -29,6 +29,15 @@
 #'   The config list is a list of parameters to pass to \code{\link[httr]{POST}}. 
 #'   Refer to documentation there for details.
 #'   
+#'   Using the settings stored in the \code{redcapConnection} object, a response
+#'   code of 408 (Request Timeout), 500 (Internal Server Error), 
+#'   502 (Bad Gateway), 503 (Service Unavailable), or 504 (Gateway Timeout)
+#'   will prompt reattempts at calling the API. See \code{\link{redcapConnection}}
+#'   for details. If the API reaches its attempt limit without resolving to 
+#'   any other code, the last response is returned. If any other response
+#'   code is returned at any point in the retry loop, the loop breaks and 
+#'   returns that response.
+#'   
 #' @author Benjamin Nutter
 #' 
 #' @references
@@ -143,25 +152,81 @@ makeApiCall <- function(rcon,
                           add = coll)
   
   checkmate::assert_list(x = body, 
+                         names = "named",
                          add = coll)
   
   checkmate::assert_list(x = config, 
+                         names = "named",
                          add = coll)
   
   checkmate::reportAssertions(coll)
   
   # Functional Code -------------------------------------------------
   
-  httr::POST(
-    url = rcon$url, 
-    body = c(list(token = rcon$token), 
-             body),
-    config = c(rcon$config, 
-               config) 
-  )
+  for (i in seq_len(rcon$retries())){
+    response <-   
+      httr::POST(url = rcon$url, 
+                 body = c(list(token = rcon$token), 
+                          body),
+                 config = c(rcon$config, 
+                            config))
+    
+    is_retry_eligible <- .makeApiCall_isRetryEligible(response = response)
+    
+    if (!is_retry_eligible) 
+      break
+    
+    # The attempt failed. Produce a message detailing the failure (when not quiet)
+    if (!rcon$retry_quietly()){
+      .makeApiCall_retryMessage(rcon = rcon, 
+                                response = response, 
+                                iteration = i)
+    }
+    
+    # Wait the designated time until trying again.
+    # when i = rcon$retries(), we've made all our attempts, we don't need to wait to exit the loop 
+    if (i < rcon$retries()) { 
+      Sys.sleep(rcon$retry_interval()[i])
+    }
+  }
+  
+  response
 }
 
-# Tests to perform --------------------------------------------------
-# *  Throw an error if \code{rcon} is not a \code{redcapApiConnection}
-# * Throw an error if \code{body} is not a list.
-# *  Throw an error if \code{config} is not a list.
+####################################################################
+# Unexported
+
+.makeApiCall_isRetryEligible <- function(response){
+  # the return from this is a logical indicating if we are ready to break the loop.
+  # we want to break the loop in cases where the response is anything that does
+  # not justify a retry. 
+  # It's somewhat silly to have this as a separate method, but it allows us
+  # to test that we can hit the retry conditions based on the status code
+  # without having to force one of these conditions onto the server. 
+  # See tests for .makeApiCall_isRetryEligible in test/testthat/test-makeApiCall.R
+  
+  retry_eligible <- response$status_code %in% c(408, 500, 502, 503, 504)
+  
+  return(retry_eligible)
+}
+
+
+
+.makeApiCall_retryMessage <- function(rcon, 
+                                      response, 
+                                      iteration){
+  msg_part1 <- sprintf("API attempt %s of %s failed. ", 
+                       iteration, 
+                       rcon$retries())
+  msg_part2 <- 
+    if (iteration < rcon$retries()){
+      sprintf("Trying again in %s seconds. ", 
+              rcon$retry_interval()[iteration])
+    } else { # when i = retries, we aren't actually going to try again. 
+      ""
+    }
+  
+  msg_part3 <- as.character(response)
+  
+  message(msg_part1, msg_part2, msg_part3)
+}
